@@ -3,10 +3,14 @@ import os
 
 import pytest
 from tox.interpreters import *  # noqa
+from tox.config import get_plugin_manager
+
 
 @pytest.fixture
 def interpreters():
-    return Interpreters()
+    pm = get_plugin_manager()
+    return Interpreters(hook=pm.hook)
+
 
 @pytest.mark.skipif("sys.platform != 'win32'")
 def test_locate_via_py(monkeypatch):
@@ -16,6 +20,7 @@ def test_locate_via_py(monkeypatch):
             assert args[1] == '-c'
             # Return value needs to actually exist!
             return sys.executable
+
     @staticmethod
     def ret_pseudopy(name):
         assert name == 'py'
@@ -24,8 +29,12 @@ def test_locate_via_py(monkeypatch):
     monkeypatch.setattr(py.path.local, 'sysfind', ret_pseudopy)
     assert locate_via_py('3', '2') == sys.executable
 
-def test_find_executable():
-    p = find_executable(sys.executable)
+
+def test_tox_get_python_executable():
+    class envconfig:
+        basepython = sys.executable
+        envname = "pyxx"
+    p = tox_get_python_executable(envconfig)
     assert p == py.path.local(sys.executable)
     for ver in [""] + "2.4 2.5 2.6 2.7 3.0 3.1 3.2 3.3".split():
         name = "python%s" % ver
@@ -38,21 +47,29 @@ def test_find_executable():
         else:
             if not py.path.local.sysfind(name):
                 continue
-        p = find_executable(name)
+        envconfig.basepython = name
+        p = tox_get_python_executable(envconfig)
         assert p
         popen = py.std.subprocess.Popen([str(p), '-V'],
-                stderr=py.std.subprocess.PIPE)
+                                        stderr=py.std.subprocess.PIPE)
         stdout, stderr = popen.communicate()
         assert ver in py.builtin._totext(stderr, "ascii")
 
+
 def test_find_executable_extra(monkeypatch):
     @staticmethod
     def sysfind(x):
         return "hello"
     monkeypatch.setattr(py.path.local, "sysfind", sysfind)
-    t = find_executable("qweqwe")
+
+    class envconfig:
+        basepython = "1lk23j"
+        envname = "pyxx"
+
+    t = tox_get_python_executable(envconfig)
     assert t == "hello"
 
+
 def test_run_and_get_interpreter_info():
     name = os.path.basename(sys.executable)
     info = run_and_get_interpreter_info(name, sys.executable)
@@ -60,33 +77,36 @@ def test_run_and_get_interpreter_info():
     assert info.name == name
     assert info.executable == sys.executable
 
+
 class TestInterpreters:
 
-    def test_get_info_self_exceptions(self, interpreters):
-        pytest.raises(ValueError, lambda:
-                      interpreters.get_info())
-        pytest.raises(ValueError, lambda:
-                      interpreters.get_info(name="12", executable="123"))
-
     def test_get_executable(self, interpreters):
-        x = interpreters.get_executable(sys.executable)
-        assert x == sys.executable
-        assert not interpreters.get_executable("12l3k1j23")
+        class envconfig:
+            basepython = sys.executable
+            envname = "pyxx"
 
-    def test_get_info__name(self, interpreters):
-        info = interpreters.get_info(executable=sys.executable)
+        x = interpreters.get_executable(envconfig)
+        assert x == sys.executable
+        info = interpreters.get_info(envconfig)
         assert info.version_info == tuple(sys.version_info)
         assert info.executable == sys.executable
         assert info.runnable
 
-    def test_get_info__name_not_exists(self, interpreters):
-        info = interpreters.get_info("qlwkejqwe")
+    def test_get_executable_no_exist(self, interpreters):
+        class envconfig:
+            basepython = "1lkj23"
+            envname = "pyxx"
+        assert not interpreters.get_executable(envconfig)
+        info = interpreters.get_info(envconfig)
         assert not info.version_info
-        assert info.name == "qlwkejqwe"
+        assert info.name == "1lkj23"
         assert not info.executable
         assert not info.runnable
 
     def test_get_sitepackagesdir_error(self, interpreters):
-        info = interpreters.get_info(sys.executable)
+        class envconfig:
+            basepython = sys.executable
+            envname = "123"
+        info = interpreters.get_info(envconfig)
         s = interpreters.get_sitepackagesdir(info, "")
         assert s
