@@ -37,6 +37,7 @@ class TestToxQuickstartMain(object):
                     'Y',         # py32
                     'Y',         # py33
                     'Y',         # py34
+                    'Y',         # py35
                     'Y',         # pypy
                     'N',         # jython
                     'py.test',   # command to run tests
@@ -54,7 +55,7 @@ class TestToxQuickstartMain(object):
 # and then run "tox" from this directory.
 
 [tox]
-envlist = py26, py27, py32, py33, py34, pypy
+envlist = py26, py27, py32, py33, py34, py35, pypy
 
 [testenv]
 commands = py.test
@@ -77,6 +78,7 @@ deps =
                     'Y',          # py32
                     'Y',          # py33
                     'Y',          # py34
+                    'Y',          # py35
                     'Y',          # pypy
                     'N',          # jython
                     'nosetests',  # command to run tests
@@ -94,7 +96,7 @@ deps =
 # and then run "tox" from this directory.
 
 [tox]
-envlist = py26, py27, py32, py33, py34, pypy
+envlist = py26, py27, py32, py33, py34, py35, pypy
 
 [testenv]
 commands = nosetests
@@ -117,6 +119,7 @@ deps =
                     'Y',          # py32
                     'Y',          # py33
                     'Y',          # py34
+                    'Y',          # py35
                     'Y',          # pypy
                     'N',          # jython
                     'trial',      # command to run tests
@@ -134,7 +137,7 @@ deps =
 # and then run "tox" from this directory.
 
 [tox]
-envlist = py26, py27, py32, py33, py34, pypy
+envlist = py26, py27, py32, py33, py34, py35, pypy
 
 [testenv]
 commands = trial
@@ -157,6 +160,7 @@ deps =
                     'Y',          # py32
                     'Y',          # py33
                     'Y',          # py34
+                    'Y',          # py35
                     'Y',          # pypy
                     'N',          # jython
                     'py.test',    # command to run tests
@@ -173,7 +177,7 @@ deps =
 # and then run "tox" from this directory.
 
 [tox]
-envlist = py26, py27, py32, py33, py34, pypy
+envlist = py26, py27, py32, py33, py34, py35, pypy
 
 [testenv]
 commands = py.test
@@ -272,7 +276,7 @@ deps =
 # and then run "tox" from this directory.
 
 [tox]
-envlist = py26, py27, py32, py33, py34, pypy, jython
+envlist = py26, py27, py32, py33, py34, py35, pypy, jython
 
 [testenv]
 commands = py.test
@@ -295,6 +299,7 @@ deps =
                     '',   # py32
                     '',   # py33
                     '',   # py34
+                    '',   # py35
                     '',   # pypy
                     '',   # jython
                     '',   # command to run tests
@@ -312,7 +317,7 @@ deps =
 # and then run "tox" from this directory.
 
 [tox]
-envlist = py26, py27, py32, py33, py34, pypy, jython
+envlist = py26, py27, py32, py33, py34, py35, pypy, jython
 
 [testenv]
 commands = {envpython} setup.py test
@@ -339,6 +344,7 @@ deps =
                     '',   # py32
                     '',   # py33
                     '',   # py34
+                    '',   # py35
                     '',   # pypy
                     '',   # jython
                     '',   # command to run tests
@@ -357,7 +363,7 @@ deps =
 # and then run "tox" from this directory.
 
 [tox]
-envlist = py26, py27, py32, py33, py34, pypy, jython
+envlist = py26, py27, py32, py33, py34, py35, pypy, jython
 
 [testenv]
 commands = {envpython} setup.py test
@@ -459,6 +465,7 @@ deps =
             'py32': True,
             'py33': True,
             'py34': True,
+            'py35': True,
             'pypy': True,
             'commands': 'nosetests -v',
             'deps': 'nose',
@@ -470,7 +477,7 @@ deps =
 # and then run "tox" from this directory.
 
 [tox]
-envlist = py27, py32, py33, py34, pypy
+envlist = py27, py32, py33, py34, py35, pypy
 
 [testenv]
 commands = nosetests -v
