@@ -4,12 +4,14 @@ from tox.result import ResultLog
 import tox
 import pytest
 
+
 @pytest.fixture
 def pkg(tmpdir):
     p = tmpdir.join("hello-1.0.tar.gz")
     p.write("whatever")
     return p
 
+
 def test_pre_set_header(pkg):
     replog = ResultLog()
     d = replog.dict
@@ -22,6 +24,7 @@ def test_pre_set_header(pkg):
     replog2 = ResultLog.loads_json(data)
     assert replog2.dict == replog.dict
 
+
 def test_set_header(pkg):
     replog = ResultLog()
     d = replog.dict
@@ -31,13 +34,15 @@ def test_set_header(pkg):
     assert replog.dict["toxversion"] == tox.__version__
     assert replog.dict["platform"] == sys.platform
     assert replog.dict["host"] == py.std.socket.getfqdn()
-    assert replog.dict["installpkg"] == {"basename": "hello-1.0.tar.gz",
-                                        "md5": pkg.computehash("md5"),
-                                        "sha256": pkg.computehash("sha256")}
+    assert replog.dict["installpkg"] == {
+        "basename": "hello-1.0.tar.gz",
+        "md5": pkg.computehash("md5"),
+        "sha256": pkg.computehash("sha256")}
     data = replog.dumps_json()
     replog2 = ResultLog.loads_json(data)
     assert replog2.dict == replog.dict
 
+
 def test_addenv_setpython(pkg):
     replog = ResultLog()
     replog.set_header(installpkg=pkg)
@@ -47,6 +52,7 @@ def test_addenv_setpython(pkg):
     assert envlog.dict["python"]["version"] == sys.version
     assert envlog.dict["python"]["executable"] == sys.executable
 
+
 def test_get_commandlog(pkg):
     replog = ResultLog()
     replog.set_header(installpkg=pkg)
@@ -60,4 +66,3 @@ def test_get_commandlog(pkg):
     assert envlog.dict["setup"]
     setuplog2 = replog.get_envlog("py26").get_commandlog("setup")
     assert setuplog2.list == setuplog.list
-
