@@ -34,7 +34,8 @@ def test_report_protocol(newconfig):
     report = session.report
     report.expect("using")
     venv = session.getvenv("mypython")
-    venv.update()
+    action = session.newaction(venv, "update")
+    venv.update(action)
     report.expect("logpopen")
 
 
@@ -301,6 +302,23 @@ def test_unknown_dep(cmd, initproj):
     ])
 
 
+def test_venv_special_chars_issue252(cmd, initproj):
+    initproj("pkg123-0.7", filedefs={
+        'tests': {'test_hello.py': "def test_hello(): pass"},
+        'tox.ini': '''
+            [tox]
+            envlist = special&&1
+            [testenv:special&&1]
+            changedir=tests
+        '''
+    })
+    result = cmd.run("tox", )
+    assert result.ret == 0
+    result.stdout.fnmatch_lines([
+        "*installed*pkg123*"
+    ])
+
+
 def test_unknown_environment(cmd, initproj):
     initproj("env123-0.7", filedefs={
         'tox.ini': ''
