@@ -51,7 +51,7 @@ def show_help(config):
             "(overridable by '-e')")
     tw.line("TOX_TESTENV_PASSENV: space-separated list of extra "
             "environment variables to be passed into test command "
-            "environemnts")
+            "environments")
 
 
 def show_help_ini(config):
@@ -211,7 +211,7 @@ class Action(object):
         if sys.platform == "win32":
             ext = os.path.splitext(str(newargs[0]))[1].lower()
             if ext == '.py' and self.venv:
-                newargs = [str(self.venv.getcommandpath())] + newargs
+                newargs = [str(self.envconfig.envpython)] + newargs
 
         return newargs
 
@@ -313,6 +313,8 @@ class Reporter(object):
 
 
 class Session:
+    """ (unstable API).  the session object that ties
+    together configuration, reporting, venv creation, testing. """
 
     def __init__(self, config, popen=subprocess.Popen, Report=Reporter):
         self.config = config
@@ -533,11 +535,11 @@ class Session:
                 action = self.newaction(venv, "envreport")
                 with action:
                     pip = venv.getcommandpath("pip")
-                    # we can't really call internal helpers here easily :/
-                    # output = venv._pcall([str(pip), "freeze"],
-                    #                      cwd=self.config.toxinidir,
-                    #                      action=action)
-                    output = py.process.cmdexec("%s freeze" % (pip))
+                    output = venv._pcall([str(pip), "freeze"],
+                                         cwd=self.config.toxinidir,
+                                         action=action)
+                    # the output contains a mime-header, skip it
+                    output = output.split("\n\n")[-1]
                     packages = output.strip().split("\n")
                     action.setactivity("installed", ",".join(packages))
                     envlog = self.resultlog.get_envlog(venv.name)
@@ -570,6 +572,9 @@ class Session:
             elif status == "platform mismatch":
                 msg = "  %s: %s" % (venv.envconfig.envname, str(status))
                 self.report.skip(msg)
+            elif status and status == "ignored failed command":
+                msg = "  %s: %s" % (venv.envconfig.envname, str(status))
+                self.report.good(msg)
             elif status and status != "skipped tests":
                 msg = "  %s: %s" % (venv.envconfig.envname, str(status))
                 self.report.error(msg)
