@@ -5,7 +5,7 @@ import re
 import codecs
 import py
 import tox
-from .config import DepConfig
+from .config import DepConfig, hookimpl
 
 
 class CreationConfig:
@@ -56,19 +56,34 @@ class VirtualEnv(object):
     def __init__(self, envconfig=None, session=None):
         self.envconfig = envconfig
         self.session = session
-        self.path = envconfig.envdir
-        self.path_config = self.path.join(".tox-config1")
+
+    @property
+    def hook(self):
+        return self.envconfig.config.pluginmanager.hook
+
+    @property
+    def path(self):
+        """ Path to environment base dir. """
+        return self.envconfig.envdir
+
+    @property
+    def path_config(self):
+        return self.path.join(".tox-config1")
 
     @property
     def name(self):
+        """ test environment name. """
         return self.envconfig.envname
 
     def __repr__(self):
         return "<VirtualEnv at %r>" % (self.path)
 
-    def getcommandpath(self, name=None, venv=True, cwd=None):
-        if name is None:
-            return self.envconfig.envpython
+    def getcommandpath(self, name, venv=True, cwd=None):
+        """ return absolute path (str or localpath) for specified
+        command name.  If it's a localpath we will rewrite it as
+        as a relative path.  If venv is True we will check if the
+        command is coming from the venv or is whitelisted to come
+        from external. """
         name = str(name)
         if os.path.isabs(name):
             return name
@@ -114,12 +129,10 @@ class VirtualEnv(object):
     def _ispython3(self):
         return "python3" in str(self.envconfig.basepython)
 
-    def update(self, action=None):
+    def update(self, action):
         """ return status string for updating actual venv to match configuration.
             if status string is empty, all is ok.
         """
-        if action is None:
-            action = self.session.newaction(self, "update")
         rconfig = CreationConfig.readconfig(self.path_config)
         if not self.envconfig.recreate and rconfig and \
            rconfig.matches(self._getliveconfig()):
@@ -130,13 +143,14 @@ class VirtualEnv(object):
         else:
             action.setactivity("recreate", self.envconfig.envdir)
         try:
-            self.create(action)
+            self.hook.tox_testenv_create(action=action, venv=self)
+            self.just_created = True
         except tox.exception.UnsupportedInterpreter:
             return sys.exc_info()[1]
         except tox.exception.InterpreterNotFound:
             return sys.exc_info()[1]
         try:
-            self.install_deps(action)
+            self.hook.tox_testenv_install_deps(action=action, venv=self)
         except tox.exception.InvocationError:
             v = sys.exc_info()[1]
             return "could not install deps %s; v = %r" % (
@@ -172,31 +186,6 @@ class VirtualEnv(object):
     def matching_platform(self):
         return re.match(self.envconfig.platform, sys.platform)
 
-    def create(self, action=None):
-        # if self.getcommandpath("activate").dirpath().check():
-        #    return
-        if action is None:
-            action = self.session.newaction(self, "create")
-
-        config_interpreter = self.getsupportedinterpreter()
-        args = [sys.executable, '-m', 'virtualenv']
-        if self.envconfig.sitepackages:
-            args.append('--system-site-packages')
-        # add interpreter explicitly, to prevent using
-        # default (virtualenv.ini)
-        args.extend(['--python', str(config_interpreter)])
-        # if sys.platform == "win32":
-        #    f, path, _ = py.std.imp.find_module("virtualenv")
-        #    f.close()
-        #    args[:1] = [str(config_interpreter), str(path)]
-        # else:
-        self.session.make_emptydir(self.path)
-        basepath = self.path.dirpath()
-        basepath.ensure(dir=1)
-        args.append(self.path.basename)
-        self._pcall(args, venv=False, action=action, cwd=basepath)
-        self.just_created = True
-
     def finish(self):
         self._getliveconfig().writeconfig(self.path_config)
 
@@ -239,15 +228,6 @@ class VirtualEnv(object):
             extraopts = ['-U', '--no-deps']
         self._install([sdistpath], extraopts=extraopts, action=action)
 
-    def install_deps(self, action=None):
-        if action is None:
-            action = self.session.newaction(self, "install_deps")
-        deps = self._getresolvedeps()
-        if deps:
-            depinfo = ", ".join(map(str, deps))
-            action.setactivity("installdeps", "%s" % depinfo)
-            self._install(deps, action=action)
-
     def _installopts(self, indexserver):
         l = []
         if indexserver:
@@ -259,7 +239,7 @@ class VirtualEnv(object):
             l.append("--pre")
         return l
 
-    def run_install_command(self, packages, options=(), action=None):
+    def run_install_command(self, packages, action, options=()):
         argv = self.envconfig.install_command[:]
         # use pip-script on win32 to avoid the executable locking
         i = argv.index('{packages}')
@@ -351,6 +331,13 @@ class VirtualEnv(object):
                     self._pcall(argv, cwd=cwd, action=action, redirect=redirect,
                                 ignore_ret=ignore_ret, testcommand=True)
                 except tox.exception.InvocationError as err:
+                    if self.envconfig.ignore_outcome:
+                        self.session.report.warning(
+                            "command failed but result from testenv is ignored\n"
+                            "  cmd: %s" % (str(err),))
+                        self.status = "ignored failed command"
+                        continue  # keep processing commands
+
                     self.session.report.error(str(err))
                     self.status = "commands failed"
                     if not self.envconfig.ignore_errors:
@@ -380,3 +367,35 @@ def getdigest(path):
     if not path.check(file=1):
         return "0" * 32
     return path.computehash()
+
+
+@hookimpl
+def tox_testenv_create(venv, action):
+    # if self.getcommandpath("activate").dirpath().check():
+    #    return
+    config_interpreter = venv.getsupportedinterpreter()
+    args = [sys.executable, '-m', 'virtualenv']
+    if venv.envconfig.sitepackages:
+        args.append('--system-site-packages')
+    # add interpreter explicitly, to prevent using
+    # default (virtualenv.ini)
+    args.extend(['--python', str(config_interpreter)])
+    # if sys.platform == "win32":
+    #    f, path, _ = py.std.imp.find_module("virtualenv")
+    #    f.close()
+    #    args[:1] = [str(config_interpreter), str(path)]
+    # else:
+    venv.session.make_emptydir(venv.path)
+    basepath = venv.path.dirpath()
+    basepath.ensure(dir=1)
+    args.append(venv.path.basename)
+    venv._pcall(args, venv=False, action=action, cwd=basepath)
+
+
+@hookimpl
+def tox_testenv_install_deps(venv, action):
+    deps = venv._getresolvedeps()
+    if deps:
+        depinfo = ", ".join(map(str, deps))
+        action.setactivity("installdeps", "%s" % depinfo)
+        venv._install(deps, action=action)
